<?php

/**
 * Classe per interagire con l'API REST di gCloud Monitoring
 *
 * @version 1.0.0
 */
require_once 'HTTP/Request2.php';

class gCloud_Monitoring {

    /**
     * @var string Attuale versione della libreria
     * @access public
     */
    const VERSION = "gCloud_Monitoring 1.0.0 [Pear]";

    /**
     * @var string nome dello stream
     * @access public
     */
    public $streamName = NULL;

    /**
     * @var string chiave di autenticazione
     * @access public
     */
    public $authentication = NULL;

    /**
     * @var string endpoint del servizio (hardcoded a "monitoring.gcloud.schema31.it)
     * @access public
     */
    public $serviceEndPoint = "monitoring.gcloud.schema31.it";

    /**
     * Variabili di appoggio per la response (http status code, header e body di risposta)
     * @access public
     */
    public $responseStatus = NULL;
    public $responseHeaders = NULL;
    public $responseBody = NULL;

    /**
     * Variabile di appoggio per l'ultimo errore ricevuto
     *
     * @var string 
     * @access public
     */
    public $lastError = "";

    public function __construct($streamName = '', $authentication = '') {

        $this->streamName = $streamName;
        $this->authentication = $authentication;
    }

    /**
     * Inizializza le variabili di stato prima di effettuare una chiamata
     * @return void
     */
    private function init() {

        $this->responseBody = NULL;
        $this->responseHeaders = NULL;
        $this->responseStatus = NULL;
        $this->lastError = NULL;
    }

    /**
     * Effettua il salvataggio di un log
     * @param string $shortMessage messaggio breve (ricercabile)
     * @param string $fullMessage testo libero (non ricercabile)
     * @param int $logLevel (ricercabile) - 0 = EMERGENCY, 1 = ALERT, 2 = CRITICAL, 3 = ERROR, 4 = WARNING, 5 = NOTICE, 6 = INFO, 7 = DEBUG
     * @param string $facility
     * @param string $file nome del file che sta richiamando questo metodo
     * @param int $line numero di riga del file che sta richiamando questo metodo
     * @param array $logMessageAdditionals array chiave / valore di parametri aggiuntivi
     * @return boolean esito operazione
     */
    public function sendLog($shortMessage = NULL, $fullMessage = NULL, $logLevel = NULL, $facility = NULL, $file = NULL, $line = NULL, $logMessageAdditionals = array()) {

        $this->init();

        /*
         * Validazione parametri obbligatori
         */
        if (strlen($shortMessage) == 0) {
            $this->lastError = "parametro 'shortMessage' non valorizzato";
            return FALSE;
        } elseif (strlen($logLevel) == 0 || !in_array($logLevel, array(0, 1, 2, 3, 4, 5, 6, 7))) {
            $this->lastError = "parametro 'logLevel' non valorizzato o non valido";
            return FALSE;
        } elseif (strlen($facility) == 0) {
            $this->lastError = "parametro 'facility' non valorizzato";
            return FALSE;
        }

        /**
         * prepariamo il body per la richiesta
         */
        $body = array(
            'host' => php_uname('n'),
            '_LibraryVersion' => self::VERSION,
            'short_message' => $shortMessage,
            'full_message' => $fullMessage,
            'level' => $logLevel,
            'facility' => $facility,
            'file' => $file,
            'line' => $line
        );

        /**
         * Aggiungiamo gli eventuali parametri aggiuntivi (accettiamo solo coppie chiave / valore)
         */
        if (is_array($logMessageAdditionals) && count($logMessageAdditionals) > 0) {
            foreach ($logMessageAdditionals as $key => $value) {
                $body["_{$key}"] = (string) $value;
            }
        }

        /*
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2("http://{$this->serviceEndPoint}/api/gelflog");
        $request->setMethod(HTTP_Request2::METHOD_POST)
                ->setHeader('AuthKey', $this->authentication)
                ->setHeader('streamName', $this->streamName)
                ->setHeader('User-Agent', self::VERSION)
                ->setHeader('Expect', "")
                ->setBody(json_encode($body));

        /*
         *  Catturiamo la risposta
         */
        $HTTPRequest2Response = $request->send();
        $this->responseBody = $HTTPRequest2Response->getBody();
        $this->responseHeaders = $HTTPRequest2Response->getHeader();
        $this->responseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if ($this->responseStatus != 201) {
            $this->lastError = "Errore esecuzione richiesta: {$this->responseStatus}";
            return FALSE;
        }

        return TRUE;
    }

}
